' SED_PBWIN - Use the PBWIN compiler
#COMPILE EXE
#DIM ALL
#INCLUDE ONCE "GLFW.INC"
#INCLUDE ONCE "GDIPUTILS.INC"

%NUMSTARS = 50

TYPE Stars
   r     AS LONG
   g     AS LONG
   b     AS LONG
   Dist  AS SINGLE
   Angle AS SINGLE
END TYPE

' =======================================================================================
' Main
' =======================================================================================
FUNCTION PBMAIN () AS LONG

   LOCAL hr, nWidth, nHeight, running, frames AS LONG
   LOCAL x, y, i, Twinkle, tp AS LONG
   LOCAL t, t0, fps AS DOUBLE
   LOCAL szTitlestr AS ASCIIZ * 200
   LOCAL strTextureData AS STRING
   LOCAL TextureWidth, TextureHeight AS LONG
   LOCAL Zoom, Tilt, Spin AS SINGLE
   DIM   glHandle AS DWORD
   DIM   Star(%NUMSTARS) AS Stars

   ' Initialize GLFW
   glfwInit

   ' Open OpenGL window
   IF ISFALSE glfwOpenWindow(640, 480, 0, 0, 0, 0, 16, 0, %GLFW_WINDOW) THEN
      glfwTerminate
      EXIT FUNCTION
   END IF

   ' Load bitmap texture from disk
   hr = GdiPlusLoadTexture("star.bmp", TextureWidth, TextureHeight, strTextureData, %TRUE)

   Zoom = -15!
   Tilt = 90.0!

   ' Assign an OpenGL handle to this texture
   glGenTextures 1, glHandle

   ' Create Linear Filtered Texture
   glBindTexture %GL_TEXTURE_2D, glHandle
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MAG_FILTER, %GL_LINEAR
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MIN_FILTER, %GL_LINEAR
   glTexImage2D %GL_TEXTURE_2D, 0, 3, TextureWidth, TextureHeight, 0, _
                %GL_RGBA, %GL_UNSIGNED_BYTE, BYVAL STRPTR(strTextureData)

   ' Enable texture mapping
   glEnable(%GL_TEXTURE_2D)
   ' Select smooth shading
   glShadeModel %GL_SMOOTH
   ' Specify clear values for the color buffers
   glClearColor 0.0!, 0.0!, 0.0!, 0.0!
   ' Specify the clear value for the depth buffer
   glClearDepth 1.0!

   ' Drop OpenGL hints how to render our graphics
   glHint %GL_POINT_SMOOTH_HINT, %GL_NICEST
   glHint %GL_LINE_SMOOTH_HINT, %GL_NICEST
   glHint %GL_POLYGON_SMOOTH_HINT, %GL_NICEST
   glHint %GL_FOG_HINT, %GL_NICEST
   glHint %GL_PERSPECTIVE_CORRECTION_HINT, %GL_NICEST

   ' Set the blending function for translucency
   glBlendFunc %GL_SRC_ALPHA,%GL_ONE
   glEnable %GL_BLEND

   FOR i = 0 TO %NUMSTARS -1
      Star(i).Angle = 0.0!
      Star(i).Dist = (i / %NUMSTARS) * 5.0! ' Calculate distance from the center
      Star(i).r = RND(0, 255)               ' Give star[loop] a random red intensity
      Star(i).g = RND(0, 255)               ' Give star[loop] a random green intensity
      Star(i).b = RND(0, 255)               ' Give star[loop] a random blue intensity
   NEXT i

   ' Enable sticky keys
   glfwEnable %GLFW_STICKY_KEYS

   ' Disable vertical sync (on cards that support it)
   glfwSwapInterval 0

   ' Main loop
   running = %TRUE
   frames = 0
   t0 = glfwGetTime

   DO WHILE running

      ' Get time and mouse position
      t = glfwGetTime
      glfwGetMousePos x, y
      ' Calculate and display FPS (frames per second)
      IF t - t0 > 1.0! OR frames = 0 THEN
         fps = frames / (t-t0)
         wsprintf szTitlestr, "NEHE Lesson 09 (%i FPS)", BYVAL fps
         glfwSetWindowTitle szTitlestr
         t0 = t
         frames = 0
      END IF
      frames = frames + 1

      '  Get window size (may be different than the requested size)
      glfwGetWindowSize nWidth, nHeight
      IF nHeight <= 0 THEN nHeight = 1

      ' Set viewport
      glViewport 0, 0, nWidth, nHeight

      ' Clear the screen and the depth buffers
      glClear %GL_COLOR_BUFFER_BIT OR %GL_DEPTH_BUFFER_BIT
      ' Select our texture
      glBindTexture %GL_TEXTURE_2D, glHandle

      ' Select and setup the projection matrix
      glMatrixMode %GL_PROJECTION
      glLoadIdentity

      ' Calculate the aspect ratio of the window
      gluPerspective 45.0!, nWidth/nHeight, 0.1!, 100.0!

      ' Select the modelview matrix
      glMatrixMode %GL_MODELVIEW

      FOR i = 0 TO %NUMSTARS - 1
         glLoadIdentity
         glTranslatef 0.0!, 0.0!, Zoom
         glRotatef Tilt, 1.0!, 0.0!, 0.0!

         glRotatef Star(i).Angle, 0.0!, 1.0!, 0.0!
         glTranslatef Star(i).Dist, 0.0!, 0.0!

         glRotatef -Star(i).Angle, 0.0!, 1.0!, 0.0!
         glRotatef -Tilt, 1.0!, 0.0!, 0.0!

         IF Twinkle THEN
            ' Assign a color using bytes
            glColor4ub Star((%NUMSTARS - i) - 1).r, Star((%NUMSTARS - i) - 1).g, Star((%NUMSTARS - i) - 1).b, 255
            glBegin %GL_QUADS
               glTexCoord2f  0.0!,  0.0!: glVertex3f -1.0!, -1.0!,  0.0!
               glTexCoord2f  1.0!,  0.0!: glVertex3f  1.0!, -1.0!,  0.0!
               glTexCoord2f  1.0!,  1.0!: glVertex3f  1.0!,  1.0!,  0.0!
               glTexCoord2f  0.0!,  1.0!: glVertex3f -1.0!,  1.0!,  0.0!
            glEnd
         END IF

         glRotatef Spin, 0.0!, 0.0!, 1.0!

         ' Assign a color using bytes
         glColor4ub Star(i).r, Star(i).g, Star(i).b, 255
         glBegin %GL_QUADS
            glTexCoord2f  0.0!,  0.0!: glVertex3f -1.0!, -1.0!,  0.0!
            glTexCoord2f  1.0!,  0.0!: glVertex3f  1.0!, -1.0!,  0.0!
            glTexCoord2f  1.0!,  1.0!: glVertex3f  1.0!,  1.0!,  0.0!
            glTexCoord2f  0.0!,  1.0!: glVertex3f -1.0!,  1.0!,  0.0!
         glEnd

         Spin = Spin + 0.01!
         Star(i).Angle = Star(i).Angle + (i / %NUMSTARS)
         Star(i).Dist = Star(i).Dist - 0.01!

         IF Star(i).Dist < 0.0! THEN
            star(i).Dist = Star(i).Dist + 5!
            star(i).r = RND(0, 255)
            star(i).g = RND(0, 255)
            star(i).b = RND(0, 255)
         END IF
      NEXT

      ' Swap buffers
      glfwSwapBuffers

      ' T key
      IF glfwGetKey(84) = %GLFW_PRESS AND ISFALSE tp THEN
         tp = %TRUE
         Twinkle = NOT Twinkle
      END IF
      IF glfwGetKey(84) <> %GLFW_PRESS THEN tp = %FALSE

      ' Are any of the arrow keys pressed?
      IF glfwGetKey(%GLFW_KEY_UP) = %GLFW_PRESS THEN Tilt = Tilt - 0.5!
      IF glfwGetKey(%GLFW_KEY_DOWN) = %GLFW_PRESS THEN Tilt = Tilt + 0.5!
      IF glfwGetKey(%GLFW_KEY_PAGEUP) = %GLFW_PRESS THEN Zoom = Zoom - 0.02!
      IF glfwGetKey(%GLFW_KEY_PAGEDOWN) = %GLFW_PRESS THEN Zoom = Zoom + 0.02!

      ' Check if the ESC key was pressed or the window was closed
      running = NOT glfwGetKey(%GLFW_KEY_ESC) AND glfwGetWindowParam(%GLFW_OPENED)

   LOOP

   ' Delete the texture
   IF glHandle THEN glDeleteTextures(1, glHandle)

   ' Close OpenGL window and terminate GLFW
   glfwTerminate

END FUNCTION
' =======================================================================================
